@@ -37,7 +37,6 @@ import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.scripts.toolkits.Gecko as Gecko
 
-from .speech_generator import SpeechGenerator
 from .spellcheck import SpellCheck
 
 _settingsManager = settings_manager.getManager()
@@ -78,10 +77,15 @@ class Script(Gecko.Script):
                 Script.togglePresentationMode,
                 cmdnames.TOGGLE_PRESENTATION_MODE)
 
-    def getSpeechGenerator(self):
-        """Returns the speech generator for this script."""
+        self.inputEventHandlers["enableStickyFocusModeHandler"] = \
+            input_event.InputEventHandler(
+                Script.enableStickyFocusMode,
+                cmdnames.SET_FOCUS_MODE_STICKY)
 
-        return SpeechGenerator(self)
+        self.inputEventHandlers["enableStickyBrowseModeHandler"] = \
+            input_event.InputEventHandler(
+                Script.enableStickyBrowseMode,
+                cmdnames.SET_BROWSE_MODE_STICKY)
 
     def getSpellCheck(self):
         """Returns the spellcheck support for this script."""
@@ -115,14 +119,6 @@ class Script(Gecko.Script):
 
         return prefs
 
-    def doWhereAmI(self, inputEvent, basicOnly):
-        """Performs the whereAmI operation."""
-
-        if self.spellcheck.isActive():
-            self.spellcheck.presentErrorDetails(not basicOnly)
-
-        super().doWhereAmI(inputEvent, basicOnly)
-
     def locusOfFocusChanged(self, event, oldFocus, newFocus):
         """Handles changes of focus of interest to the script."""
 
@@ -135,7 +131,7 @@ class Script(Gecko.Script):
         super().locusOfFocusChanged(event, oldFocus, newFocus)
 
     def useFocusMode(self, obj):
-        if self.isEditableMessage(obj):
+        if self.utilities.isEditableMessage(obj):
             msg = "THUNDERBIRD: Using focus mode for editable message %s" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
             return True
@@ -144,8 +140,20 @@ class Script(Gecko.Script):
         debug.println(debug.LEVEL_INFO, msg, True)
         return super().useFocusMode(obj)
 
+    def enableStickyBrowseMode(self, inputEvent, forceMessage=False):
+        if self.utilities.isEditableMessage(orca_state.locusOfFocus):
+            return
+
+        super().enableStickyBrowseMode(inputEvent, forceMessage)
+
+    def enableStickyFocusMode(self, inputEvent, forceMessage=False):
+        if self.utilities.isEditableMessage(orca_state.locusOfFocus):
+            return
+
+        super().enableStickyFocusMode(inputEvent, forceMessage)
+
     def togglePresentationMode(self, inputEvent):
-        if self._inFocusMode and self.isEditableMessage(orca_state.locusOfFocus):
+        if self._inFocusMode and self.utilities.isEditableMessage(orca_state.locusOfFocus):
             return
 
         super().togglePresentationMode(inputEvent)
@@ -153,7 +161,7 @@ class Script(Gecko.Script):
     def useStructuralNavigationModel(self):
         """Returns True if structural navigation should be enabled here."""
 
-        if self.isEditableMessage(orca_state.locusOfFocus):
+        if self.utilities.isEditableMessage(orca_state.locusOfFocus):
             return False
 
         return super().useStructuralNavigationModel()
@@ -165,8 +173,6 @@ class Script(Gecko.Script):
             return
 
         self._lastAutoComplete = ""
-        self.pointOfReference['lastAutoComplete'] = None
-
         obj = event.source
         if self.spellcheck.isAutoFocusEvent(event):
             orca.setLocusOfFocus(event, event.source, False)
@@ -176,7 +182,7 @@ class Script(Gecko.Script):
             super().onFocusedChanged(event)
             return
 
-        if self.isEditableMessage(obj):
+        if self.utilities.isEditableMessage(obj):
             super().onFocusedChanged(event)
             return
 
@@ -185,6 +191,12 @@ class Script(Gecko.Script):
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
+        if self.utilities.isEditableMessage(event.source):
+            return
+
+        if self.inFocusMode():
+            return
+
         obj = event.source
         if self.utilities.isDocument(obj) and not event.detail1:
             try:
@@ -203,7 +215,7 @@ class Script(Gecko.Script):
     def onCaretMoved(self, event):
         """Callback for object:text-caret-moved accessibility events."""
 
-        if self.isEditableMessage(event.source):
+        if self.utilities.isEditableMessage(event.source):
             if event.detail1 == -1:
                 return
             self.spellcheck.setDocumentPosition(event.source, event.detail1)
@@ -286,7 +298,7 @@ class Script(Gecko.Script):
 
         # Try to stop unwanted chatter when a message is being replied to.
         # See bgo#618484.
-        if isSystemEvent and self.isEditableMessage(obj):
+        if isSystemEvent and self.utilities.isEditableMessage(obj):
             return
 
         # Speak the autocompleted text, but only if it is different
@@ -310,7 +322,6 @@ class Script(Gecko.Script):
             if hasSelection or isSystemEvent:
                 speech.speak(event.any_data)
                 self._lastAutoComplete = event.any_data
-                self.pointOfReference['lastAutoComplete'] = hash(obj)
                 return
 
         super().onTextInserted(event)
@@ -323,7 +334,7 @@ class Script(Gecko.Script):
         if obj == spellCheckEntry:
             return
 
-        if self.isEditableMessage(obj) and self.spellcheck.isActive():
+        if self.utilities.isEditableMessage(obj) and self.spellcheck.isActive():
             text = obj.queryText()
             selStart, selEnd = text.getSelection(0)
             self.spellcheck.setDocumentPosition(obj, selStart)
@@ -373,17 +384,6 @@ class Script(Gecko.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             self.sayAll(None)
 
-    def sayCharacter(self, obj):
-        """Speaks the character at the current caret position."""
-
-        if self.isEditableMessage(obj):
-            text = self.utilities.queryNonEmptyText(obj)
-            if text and text.caretOffset + 1 >= text.characterCount:
-                default.Script.sayCharacter(self, obj)
-                return
-
-        super().sayCharacter(obj)
-
     def sayWord(self, obj):
         """Speaks the word at the current caret position."""
 
@@ -407,24 +407,6 @@ class Script(Gecko.Script):
 
         return default.Script.toggleFlatReviewMode(self, inputEvent)
 
-    def isEditableMessage(self, obj):
-        """Returns True if this is a editable message."""
-
-        if not obj:
-            return False
-
-        if not obj.getState().contains(pyatspi.STATE_EDITABLE):
-            return False
-
-        if self.utilities.isDocument(obj):
-            return True
-
-        document = self.utilities.getContainingDocument(obj)
-        if document and document.getState().contains(pyatspi.STATE_EDITABLE):
-            return True
-
-        return False
-
     def onWindowActivated(self, event):
         """Callback for window:activate accessibility events."""
 
